package cn.iocoder.yudao.module.infra.service.minio;

import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.infra.config.MinioConfigProperties;
import io.minio.BucketExistsArgs;
import io.minio.GetPresignedObjectUrlArgs;
import io.minio.MakeBucketArgs;
import io.minio.MinioClient;
import io.minio.PostPolicy;
import io.minio.PutObjectArgs;
import io.minio.RemoveObjectArgs;
import io.minio.StatObjectArgs;
import io.minio.StatObjectResponse;
import io.minio.http.Method;
import io.minio.ComposeObjectArgs;
import io.minio.ComposeSource;
import io.minio.RemoveObjectsArgs;
import io.minio.messages.DeleteObject;
import cn.iocoder.yudao.module.infra.controller.admin.file.vo.file.MultipartUploadCompleteRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import java.io.InputStream;
import java.time.Instant;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * MinIO 文件服务类
 *
 * @author AI Assistant
 */
@Slf4j
@Service
public class MinioService {

    @Resource
    private MinioConfigProperties minioConfigProperties;

    private MinioClient minioClient;
    
    // 用于并行处理的线程池
    private ExecutorService executorService;

    /**
     * 初始化 MinIO 客户端
     */
    @PostConstruct
    public void init() {
        try {
            this.minioClient = MinioClient.builder()
                    .endpoint(minioConfigProperties.getEndpoint())
                    .credentials(minioConfigProperties.getAccessKey(), minioConfigProperties.getSecretKey())
                    .build();

            // 初始化线程池，用于并行处理（使用配置化的线程池大小）
            int corePoolSize = minioConfigProperties.getMultipart().getCorePoolSize();
            int maxPoolSize = minioConfigProperties.getMultipart().getMaxPoolSize();
            this.executorService = Executors.newFixedThreadPool(maxPoolSize, r -> {
                Thread t = new Thread(r, "minio-parallel-pool");
                t.setDaemon(true);
                t.setPriority(Thread.NORM_PRIORITY + 1); // 提高优先级
                return t;
            });

            // 检查桶是否存在，不存在则创建
            String[] bucketsToCreate = {minioConfigProperties.getBucketName(), "user-uploads"};

            for (String bucketName : bucketsToCreate) {
                boolean bucketExists = minioClient.bucketExists(
                        BucketExistsArgs.builder()
                                .bucket(bucketName)
                                .build()
                );

                if (!bucketExists) {
                    minioClient.makeBucket(
                            MakeBucketArgs.builder()
                                    .bucket(bucketName)
                                    .build()
                    );
                    log.info("创建存储桶: {}", bucketName);
                }
            }

            log.info("MinIO 客户端初始化成功. Endpoint: {}, Bucket: {}",
                    minioConfigProperties.getEndpoint(), minioConfigProperties.getBucketName());
        } catch (Exception e) {
            log.error("MinIO 客户端初始化失败", e);
            throw new RuntimeException("MinIO 客户端初始化失败", e);
        }
    }

    /**
     * 销毁方法，关闭线程池
     */
    @PreDestroy
    public void destroy() {
        if (executorService != null && !executorService.isShutdown()) {
            executorService.shutdown();
            log.info("MinIO 线程池已关闭");
        }
    }

    /**
     * 上传单个文件
     *
     * @param file 上传的文件
     * @param path 文件存储路径，如果为空则自动生成
     * @return 文件访问URL
     */
    public String uploadFile(MultipartFile file, String path) throws Exception {
        if (file.isEmpty()) {
            throw new IllegalArgumentException("上传文件不能为空");
        }

        // 如果路径为空，则生成随机路径
        if (path == null || path.trim().isEmpty()) {
            String originalFilename = file.getOriginalFilename();
            String extension = "";
            if (originalFilename != null && originalFilename.contains(".")) {
                extension = originalFilename.substring(originalFilename.lastIndexOf("."));
            }
            path = UUID.randomUUID().toString() + extension;
        }

        try (InputStream inputStream = file.getInputStream()) {
            // 上传文件到 MinIO
            minioClient.putObject(
                    PutObjectArgs.builder()
                            .bucket(minioConfigProperties.getBucketName())
                            .object(path)
                            .stream(inputStream, file.getSize(), -1)
                            .contentType(file.getContentType())
                            .build()
            );

            // 生成文件访问URL
            String fileUrl = minioConfigProperties.getEndpoint() + "/" +
                           minioConfigProperties.getBucketName() + "/" + path;

            log.info("文件上传成功: {}", fileUrl);
            return fileUrl;
        } catch (Exception e) {
            log.error("文件上传失败: {}", path, e);
            throw new Exception("文件上传失败: " + e.getMessage(), e);
        }
    }

    /**
     * 上传多个文件
     *
     * @param files 上传的文件数组
     * @return 文件访问URL列表
     */
    public List<String> uploadFiles(MultipartFile[] files) throws Exception {
        if (files == null || files.length == 0) {
            throw new IllegalArgumentException("上传文件不能为空");
        }

        List<String> fileUrls = new ArrayList<>();

        for (MultipartFile file : files) {
            if (!file.isEmpty()) {
                String fileUrl = uploadFile(file, null);
                fileUrls.add(fileUrl);
            }
        }

        log.info("批量文件上传完成，共上传 {} 个文件", fileUrls.size());
        return fileUrls;
    }

    /**
     * 删除文件
     *
     * @param path 文件路径
     */
    public void deleteFile(String path) throws Exception {
        try {
            minioClient.removeObject(
                    RemoveObjectArgs.builder()
                            .bucket(minioConfigProperties.getBucketName())
                            .object(path)
                            .build()
            );
            log.info("文件删除成功: {}", path);
        } catch (Exception e) {
            log.error("文件删除失败: {}", path, e);
            throw new Exception("文件删除失败: " + e.getMessage(), e);
        }
    }

    /**
     * 生成预签名上传凭证
     *
     * @param fileName 文件名
     * @param fileType 文件类型
     * @return 预签名上传表单数据
     */
    public Map<String, Object> generatePresignedUploadCredentials(String fileName, String fileType) {
        try {
            // 1. 安全验证 - 从SecurityContext获取当前用户
            Long userId = SecurityFrameworkUtils.getLoginUserId();
            if (userId == null) {
                throw new IllegalStateException("用户未登录");
            }

            // 2. 生成唯一对象名
            String sanitizedFileName = sanitizeFilename(fileName);
            String objectName = String.format("user_%d/%s_%s", userId, UUID.randomUUID(), sanitizedFileName);

            // 3. 创建PostPolicy
            // 使用固定的 user-uploads bucket 用于用户上传
            String bucketName = "user-uploads";
            PostPolicy policy = new PostPolicy(bucketName, ZonedDateTime.now().plusHours(1));

            // 设置对象名
            policy.addEqualsCondition("key", objectName);

            // 添加文件大小限制：最小1字节，最大10GB
            policy.addContentLengthRangeCondition(1, 10L * 1024 * 1024 * 1024);

            // 添加Content-Type条件
            policy.addEqualsCondition("Content-Type", fileType);

            // 4. 调用MinIO客户端获取预签名表单数据
            Map<String, String> formData = minioClient.getPresignedPostFormData(policy);

            // 5. 构建返回结构
            Map<String, Object> result = new java.util.HashMap<>();
            result.put("url", minioConfigProperties.getEndpoint() + "/" + bucketName);
            result.put("fields", formData);
            result.put("objectName", objectName);

            log.info("生成预签名上传凭证成功，用户ID: {}, 对象名: {}", userId, objectName);
            return result;

        } catch (Exception e) {
            log.error("生成预签名上传凭证失败", e);
            throw new RuntimeException("生成预签名上传凭证失败: " + e.getMessage(), e);
        }
    }

    /**
     * 文件名安全处理
     *
     * @param fileName 原文件名
     * @return 安全的文件名
     */
    private String sanitizeFilename(String fileName) {
        if (fileName == null || fileName.trim().isEmpty()) {
            return "unknown";
        }
        return fileName.replaceAll("[^a-zA-Z0-9.-]", "_");
    }

    /**
     * 生成预签名下载URL
     *
     * @param objectName 对象名称
     * @return 包含下载URL等信息的Map
     */
    public Map<String, Object> generatePresignedDownloadUrl(String objectName) {
        try {
            // 1. 参数验证
            if (objectName == null || objectName.trim().isEmpty()) {
                throw new IllegalArgumentException("对象名称不能为空");
            }
            // 3. 验证文件是否存在
            String bucketName = "user-uploads";
            try {
                StatObjectResponse stat = minioClient.statObject(
                    StatObjectArgs.builder()
                        .bucket(bucketName)
                        .object(objectName)
                        .build()
                );
                log.debug("文件存在检查通过: {}, 大小: {}", objectName, stat.size());
            } catch (Exception e) {
                log.warn("文件不存在: {}", objectName);
                throw new RuntimeException("文件不存在");
            }

            // 4. 生成预签名下载URL
            String downloadUrl = minioClient.getPresignedObjectUrl(
                GetPresignedObjectUrlArgs.builder()
                    .method(Method.GET)
                    .bucket(bucketName)
                    .object(objectName)
                    .build()
            );

            // 6. 构建返回结果
            Map<String, Object> result = new java.util.HashMap<>();
            result.put("objectName", objectName);
            result.put("downloadUrl", downloadUrl);

            log.info("生成预签名下载URL成功，对象名: {}", objectName);
            return result;

        } catch (Exception e) {
            log.error("生成预签名下载URL失败，对象名: {}", objectName, e);
            throw new RuntimeException("生成预签名下载URL失败: " + e.getMessage(), e);
        }
    }

    /**
     * 初始化分片上传（简化实现）
     * 注意：此实现仅做会话记录，实际分片由前端直接上传到MinIO
     *
     * @param uploadId 上传会话ID
     * @param objectName 对象名称
     * @return 上传会话ID
     */
    public String initMultipartUpload(String uploadId, String objectName) throws Exception {
        try {
            // 验证bucket是否存在
            String bucketName = "user-uploads";
            boolean bucketExists = minioClient.bucketExists(
                BucketExistsArgs.builder()
                    .bucket(bucketName)
                    .build()
            );

            if (!bucketExists) {
                throw new Exception("存储桶不存在: " + bucketName);
            }

            log.info("初始化分片上传成功，上传会话ID: {}, 对象名: {}", uploadId, objectName);
            return uploadId;
        } catch (Exception e) {
            log.error("初始化分片上传失败", e);
            throw new Exception("初始化分片上传失败: " + e.getMessage(), e);
        }
    }

    /**
     * 生成分片预签名URL
     *
     * @param uploadId 上传会话ID
     * @param objectName 对象名称
     * @param partNumber 分片编号
     * @param expires 有效期（秒）
     * @return 预签名URL
     */
    public String generateChunkPresignedUrl(String uploadId, String objectName, Integer partNumber, Integer expires) throws Exception {
        try {
            // 使用固定的 user-uploads bucket
            String bucketName = "user-uploads";

            // 生成临时分片对象名
            String chunkObjectName = objectName + ".part" + partNumber;

            String presignedUrl = minioClient.getPresignedObjectUrl(
                GetPresignedObjectUrlArgs.builder()
                    .method(Method.PUT)
                    .bucket(bucketName)
                    .object(chunkObjectName)
                    .expiry(expires)
                    .build()
            );

            log.info("生成分片预签名URL成功，分片编号: {}", partNumber);
            return presignedUrl;
        } catch (Exception e) {
            log.error("生成分片预签名URL失败", e);
            throw new Exception("生成分片预签名URL失败: " + e.getMessage(), e);
        }
    }

    /**
     * 分片上传完成策略枚举
     */
    private enum CompletionStrategy {
        STANDARD("标准快速模式"),
        TURBO("极速模式"),
        INSTANT("立即响应模式");

        private final String description;

        CompletionStrategy(String description) {
            this.description = description;
        }

        public String getDescription() {
            return description;
        }
    }

    /**
     * 完成分片上传 - 优化版本（统一处理逻辑）
     * 根据分片数量自动选择最优策略
     *
     * @param uploadId 上传会话ID
     * @param objectName 对象名称
     * @param parts 分片信息列表
     * @return 完成结果
     */
    public MultipartUploadCompleteResult completeMultipartUpload(String uploadId, String objectName,
                                                                java.util.List<MultipartUploadCompleteRequest.PartInfo> parts) throws Exception {
        // 根据分片数量和配置选择策略
        int instantThreshold = minioConfigProperties.getMultipart().getInstantModeThreshold();
        int turboThreshold = minioConfigProperties.getMultipart().getTurboModeThreshold();
        
        CompletionStrategy strategy;
        if (parts.size() > instantThreshold) {
            strategy = CompletionStrategy.INSTANT;
        } else if (parts.size() > turboThreshold) {
            strategy = CompletionStrategy.TURBO;
        } else {
            strategy = CompletionStrategy.STANDARD;
        }
        
        return executeMultipartUploadCompletion(uploadId, objectName, parts, strategy);
    }

    /**
     * 统一执行分片上传完成逻辑
     * 根据策略选择不同的处理方式
     */
    private MultipartUploadCompleteResult executeMultipartUploadCompletion(String uploadId, String objectName,
                                                                          java.util.List<MultipartUploadCompleteRequest.PartInfo> parts, 
                                                                          CompletionStrategy strategy) throws Exception {
        long startTime = System.currentTimeMillis();
        String bucketName = "user-uploads";
        
        try {
            log.info("开始{}处理{}个分片文件", strategy.getDescription(), parts.size());
            
            // 1. 通用预处理：排序分片
            parts.sort((a, b) -> a.getPartNumber().compareTo(b.getPartNumber()));
            
            // 2. 构建合并源列表
            List<ComposeSource> sources = buildComposeSources(bucketName, objectName, parts);
            
            // 3. 根据策略选择处理方式
            switch (strategy) {
                case INSTANT:
                    return executeInstantMode(bucketName, objectName, parts, sources, startTime);
                case TURBO:
                    return executeTurboMode(bucketName, objectName, parts, sources, startTime);
                case STANDARD:
                default:
                    return executeStandardMode(bucketName, objectName, parts, sources, startTime);
            }
            
        } catch (Exception e) {
            log.error("{}失败: {}", strategy.getDescription(), e.getMessage());
            throw new Exception(strategy.getDescription() + "失败: " + e.getMessage(), e);
        }
    }

    /**
     * 构建合并源列表
     */
    private List<ComposeSource> buildComposeSources(String bucketName, String objectName, 
                                                   java.util.List<MultipartUploadCompleteRequest.PartInfo> parts) {
        return parts.stream()
            .map(part -> ComposeSource.builder()
                .bucket(bucketName)
                .object(objectName + ".part" + part.getPartNumber())
                .build())
            .collect(Collectors.toList());
    }

    /**
     * 执行立即响应模式
     */
    private MultipartUploadCompleteResult executeInstantMode(String bucketName, String objectName,
                                                           java.util.List<MultipartUploadCompleteRequest.PartInfo> parts,
                                                           List<ComposeSource> sources, long startTime) {
        // 1. 预生成文件URL（基于MinIO的规则）
        String fileUrl = minioConfigProperties.getEndpoint() + "/" + bucketName + "/" + objectName;
        
        // 2. 生成临时ETag
        String etag = "\"" + UUID.randomUUID().toString().replace("-", "") + "\"";
        
        // 3. 异步执行合并操作
        CompletableFuture.runAsync(() -> {
            try {
                long asyncStartTime = System.currentTimeMillis();
                log.info("开始异步合并{}个分片文件", parts.size());
                
                // 执行合并
                minioClient.composeObject(
                    ComposeObjectArgs.builder()
                        .bucket(bucketName)
                        .object(objectName)
                        .sources(sources)
                        .build()
                );
                
                // 异步清理分片文件
                cleanupChunkFilesBatch(bucketName, objectName, parts);
                
                long asyncTotalTime = System.currentTimeMillis() - asyncStartTime;
                log.info("异步合并完成，耗时: {}ms", asyncTotalTime);
                
            } catch (Exception e) {
                log.error("异步合并分片文件失败", e);
            }
        }, executorService);
        
        long totalTime = System.currentTimeMillis() - startTime;
        log.info("立即响应模式完成，响应耗时: {}ms", totalTime);
        
        return new MultipartUploadCompleteResult(fileUrl, etag);
    }

    /**
     * 执行极速模式 - 借鉴分批并行处理思想
     */
    private MultipartUploadCompleteResult executeTurboMode(String bucketName, String objectName,
                                                         java.util.List<MultipartUploadCompleteRequest.PartInfo> parts,
                                                         List<ComposeSource> sources, long startTime) throws Exception {
        // 1. 异步预生成URL（不阻塞后续操作）
        CompletableFuture<String> urlFuture = CompletableFuture.supplyAsync(() -> {
            try {
                return minioClient.getPresignedObjectUrl(
                    GetPresignedObjectUrlArgs.builder()
                        .method(Method.GET)
                        .bucket(bucketName)
                        .object(objectName)
                        .build()
                );
            } catch (Exception e) {
                // 如果预生成URL失败，使用基本URL规则
                return minioConfigProperties.getEndpoint() + "/" + bucketName + "/" + objectName;
            }
        }, executorService);
        
        long preProcessTime = System.currentTimeMillis();
        log.info("预处理完成，耗时: {}ms", preProcessTime - startTime);
        
        // 2. 判断是否需要分批处理（超大文件优化）
        if (sources.size() > 100) {
            // 分批并行合并策略
            executeBatchCompose(bucketName, objectName, sources);
        } else {
            // 标准合并
            minioClient.composeObject(
                ComposeObjectArgs.builder()
                    .bucket(bucketName)
                    .object(objectName)
                    .sources(sources)
                    .build()
            );
        }
        
        long composeTime = System.currentTimeMillis();
        log.info("文件合并完成，耗时: {}ms", composeTime - preProcessTime);
        
        // 3. 异步清理
        cleanupChunkFilesBatch(bucketName, objectName, parts);
        
        // 4. 获取预生成的URL（使用配置化的超时时间）
        String fileUrl = getUrlWithTimeout(urlFuture, bucketName, objectName);
        
        long totalTime = System.currentTimeMillis();
        log.info("极速模式完成，总耗时: {}ms", totalTime - startTime);
        
        // 5. 返回结果
        String etag = "\"" + UUID.randomUUID().toString().replace("-", "") + "\"";
        return new MultipartUploadCompleteResult(fileUrl, etag);
    }

    /**
     * 执行标准模式
     */
    private MultipartUploadCompleteResult executeStandardMode(String bucketName, String objectName,
                                                            java.util.List<MultipartUploadCompleteRequest.PartInfo> parts,
                                                            List<ComposeSource> sources, long startTime) throws Exception {
        // 1. 并行预生成URL和合并文件
        CompletableFuture<String> urlFuture = CompletableFuture.supplyAsync(() -> {
            try {
                return minioClient.getPresignedObjectUrl(
                    GetPresignedObjectUrlArgs.builder()
                        .method(Method.GET)
                        .bucket(bucketName)
                        .object(objectName)
                        .build()
                );
            } catch (Exception e) {
                throw new RuntimeException("预生成URL失败", e);
            }
        }, executorService);

        // 2. 直接合并分片文件
        minioClient.composeObject(
            ComposeObjectArgs.builder()
                .bucket(bucketName)
                .object(objectName)
                .sources(sources)
                .build()
        );

        long composeTime = System.currentTimeMillis();
        log.info("分片文件合并完成，耗时: {}ms", composeTime - startTime);

        // 3. 并行获取文件信息和URL
        CompletableFuture<StatObjectResponse> statFuture = CompletableFuture.supplyAsync(() -> {
            try {
                return minioClient.statObject(
                    StatObjectArgs.builder()
                        .bucket(bucketName)
                        .object(objectName)
                        .build()
                );
            } catch (Exception e) {
                throw new RuntimeException("获取文件信息失败", e);
            }
        }, executorService);

        // 4. 异步批量清理分片文件（不阻塞响应）
        cleanupChunkFilesBatch(bucketName, objectName, parts);

        // 5. 获取结果（超时时间缩短到5秒）
        String fileUrl = urlFuture.get(5, TimeUnit.SECONDS);
        StatObjectResponse finalStat = statFuture.get(5, TimeUnit.SECONDS);
        String etag = finalStat.etag();

        long totalTime = System.currentTimeMillis();
        log.info("标准模式完成，对象名: {}, 文件大小: {}, 总耗时: {}ms", 
                 objectName, finalStat.size(), totalTime - startTime);
        
        return new MultipartUploadCompleteResult(fileUrl, etag);
    }

    /**
     * 获取URL（带超时处理）
     */
    private String getUrlWithTimeout(CompletableFuture<String> urlFuture, String bucketName, String objectName) {
        int urlTimeout = minioConfigProperties.getMultipart().getUrlGenerateTimeout();
        try {
            return urlFuture.get(urlTimeout, TimeUnit.SECONDS);
        } catch (Exception e) {
            log.warn("获取预生成URL超时（{}秒），使用基本URL", urlTimeout);
            return minioConfigProperties.getEndpoint() + "/" + bucketName + "/" + objectName;
        }
    }

    /**
     * 分批并行合并策略 - 借鉴优秀实践
     * 将大量分片分批处理，减少单次操作的复杂度
     */
    private void executeBatchCompose(String bucketName, String objectName, List<ComposeSource> sources) throws Exception {
        int batchSize = minioConfigProperties.getMultipart().getBatchSize();
        log.info("启用分批合并策略，总分片数: {}, 批次大小: {}", sources.size(), batchSize);
        
        if (sources.size() <= batchSize) {
            // 如果分片数量不超过批次大小，直接合并
            minioClient.composeObject(
                ComposeObjectArgs.builder()
                    .bucket(bucketName)
                    .object(objectName)
                    .sources(sources)
                    .build()
            );
            return;
        }

        // 1. 分批创建临时合并对象
        List<String> tempObjects = new ArrayList<>();
        List<CompletableFuture<Void>> batchFutures = new ArrayList<>();
        
        for (int i = 0; i < sources.size(); i += batchSize) {
            int endIndex = Math.min(i + batchSize, sources.size());
            List<ComposeSource> batch = sources.subList(i, endIndex);
            String tempObjectName = objectName + ".temp." + (i / batchSize);
            tempObjects.add(tempObjectName);
            
            // 并行处理每个批次
            CompletableFuture<Void> batchFuture = CompletableFuture.runAsync(() -> {
                try {
                    long batchStartTime = System.currentTimeMillis();
                    minioClient.composeObject(
                        ComposeObjectArgs.builder()
                            .bucket(bucketName)
                            .object(tempObjectName)
                            .sources(batch)
                            .build()
                    );
                    long batchTime = System.currentTimeMillis() - batchStartTime;
                    log.debug("批次合并完成: {}, 耗时: {}ms, 分片数: {}", 
                             tempObjectName, batchTime, batch.size());
                } catch (Exception e) {
                    log.error("批次合并失败: {}", tempObjectName, e);
                    throw new RuntimeException("批次合并失败: " + tempObjectName, e);
                }
            }, executorService);
            
            batchFutures.add(batchFuture);
        }
        
        // 2. 等待所有批次完成
        try {
            CompletableFuture.allOf(batchFutures.toArray(new CompletableFuture[0])).get();
            log.info("所有批次合并完成，共{}个批次", tempObjects.size());
        } catch (Exception e) {
            log.error("批次合并过程中出现错误", e);
            // 清理已创建的临时对象
            cleanupTempObjects(bucketName, tempObjects);
            throw new Exception("分批合并失败", e);
        }
        
        // 3. 最终合并临时对象
        try {
            List<ComposeSource> tempSources = tempObjects.stream()
                .map(tempObj -> ComposeSource.builder()
                    .bucket(bucketName)
                    .object(tempObj)
                    .build())
                .collect(Collectors.toList());
            
            minioClient.composeObject(
                ComposeObjectArgs.builder()
                    .bucket(bucketName)
                    .object(objectName)
                    .sources(tempSources)
                    .build()
            );
            
            log.info("最终合并完成，临时对象数: {}", tempObjects.size());
            
        } finally {
            // 4. 清理临时对象
            cleanupTempObjects(bucketName, tempObjects);
        }
    }

    /**
     * 清理临时对象
     */
    private void cleanupTempObjects(String bucketName, List<String> tempObjects) {
        if (tempObjects.isEmpty()) {
            return;
        }
        
        CompletableFuture.runAsync(() -> {
            try {
                List<DeleteObject> deleteObjects = tempObjects.stream()
                    .map(DeleteObject::new)
                    .collect(Collectors.toList());
                
                minioClient.removeObjects(
                    RemoveObjectsArgs.builder()
                        .bucket(bucketName)
                        .objects(deleteObjects)
                        .build()
                );
                
                log.info("临时对象清理完成，清理数量: {}", tempObjects.size());
            } catch (Exception e) {
                log.error("清理临时对象失败", e);
            }
        }, executorService);
    }







    /**
     * 异步批量清理分片文件
     * 使用MinIO批量删除API，性能更优
     */
    private void cleanupChunkFilesBatch(String bucketName, String objectName, 
                                       List<MultipartUploadCompleteRequest.PartInfo> parts) {
        CompletableFuture.runAsync(() -> {
            try {
                log.info("开始异步批量清理{}个分片文件", parts.size());
                
                // 构建待删除对象列表
                List<DeleteObject> deleteObjects = parts.stream()
                    .map(part -> {
                        String chunkObjectName = objectName + ".part" + part.getPartNumber();
                        return new DeleteObject(chunkObjectName);
                    })
                    .collect(Collectors.toList());

                // 批量删除分片文件
                if (!deleteObjects.isEmpty()) {
                    minioClient.removeObjects(
                        RemoveObjectsArgs.builder()
                            .bucket(bucketName)
                            .objects(deleteObjects)
                            .build()
                    );
                    log.info("异步批量清理分片文件完成，清理了{}个文件", deleteObjects.size());
                }
                
            } catch (Exception e) {
                log.error("异步批量清理分片文件失败", e);
                // 如果批量删除失败，降级为单个删除
                fallbackToSingleDelete(bucketName, objectName, parts);
            }
        }, executorService);
    }

    /**
     * 降级为单个删除（当批量删除失败时）
     */
    private void fallbackToSingleDelete(String bucketName, String objectName, 
                                       List<MultipartUploadCompleteRequest.PartInfo> parts) {
        log.warn("降级为单个删除模式");
        parts.parallelStream().forEach(part -> {
            String chunkObjectName = objectName + ".part" + part.getPartNumber();
            try {
                minioClient.removeObject(
                    RemoveObjectArgs.builder()
                        .bucket(bucketName)
                        .object(chunkObjectName)
                        .build()
                );
                log.debug("降级删除分片文件: {}", chunkObjectName);
            } catch (Exception e) {
                log.warn("降级删除分片文件失败: {}", chunkObjectName, e);
            }
        });
    }

    /**
     * 取消分片上传 - 清理已上传的分片文件
     * 删除所有已上传的分片文件
     *
     * @param uploadId 上传会话ID
     * @param objectName 对象名称
     * @param totalChunks 总分片数
     */
    public void abortMultipartUpload(String uploadId, String objectName, Integer totalChunks) throws Exception {
        try {
            String bucketName = "user-uploads";

            // 清理可能存在的分片文件
            // 如果有totalChunks信息，则使用精确清理，否则尝试删除最多1000个分片文件
            int maxParts = totalChunks != null ? totalChunks : 1000;
            int cleanedCount = 0;

            for (int partNumber = 1; partNumber <= maxParts; partNumber++) {
                String chunkObjectName = objectName + ".part" + partNumber;

                try {
                    // 检查分片文件是否存在
                    minioClient.statObject(
                        StatObjectArgs.builder()
                            .bucket(bucketName)
                            .object(chunkObjectName)
                            .build()
                    );

                    // 如果存在，则删除
                    minioClient.removeObject(
                        RemoveObjectArgs.builder()
                            .bucket(bucketName)
                            .object(chunkObjectName)
                            .build()
                    );

                    cleanedCount++;
                    log.debug("清理分片文件: {}", chunkObjectName);

                } catch (Exception e) {
                    // 分片文件不存在，跳过
                    log.debug("分片文件不存在，跳过: {}", chunkObjectName);
                }
            }

            log.info("取消分片上传成功，上传会话ID: {}, 对象名: {}, 清理了{}个分片文件",
                     uploadId, objectName, cleanedCount);
        } catch (Exception e) {
            log.error("取消分片上传失败", e);
            throw new Exception("取消分片上传失败: " + e.getMessage(), e);
        }
    }

    /**
     * 分片上传完成结果
     */
    public static class MultipartUploadCompleteResult {
        private final String fileUrl;
        private final String etag;

        public MultipartUploadCompleteResult(String fileUrl, String etag) {
            this.fileUrl = fileUrl;
            this.etag = etag;
        }

        public String getFileUrl() {
            return fileUrl;
        }

        public String getEtag() {
            return etag;
        }
    }

}
